/**
 * 
 * $Id: GPVehicle.java 627 2017-10-30 11:21:12Z jjct $
 *
 * HISTORIQUE
* VERSION:2.1:DM:DM-630:11/10/2019:[GENOPUS] Modifications widgets pour FT 336 de PSIMU
* VERSION:2.1:DM:DM-635:11/10/2019:[GENOPUS] Montee de niveau GENIUS 1.9
 * 
 * VERSION:1.2.1:DM:DM-161:29/12/2016:Class creation.
 * VERSION:1.3:DM:DM-165:04/07/2017:PATRIUS upgrade to V3.3. Removed exception in getAssembly if mass < 0.
 * VERSION:1.3:DM:DM-193:27/09/2017:Copy/paste menu active by default.
 * VERSION:2.0:DM:DM-213:31/05/2018:Upgrade to PATRIUS 4.1 and remove GENOPUS Custom classes
 * VERSION:2.0:DM:DM-218:13/09/2018:Upgrade to GENIUS 1.8
 * 
 * FIN-HISTORIQUE
 *
 * This software is part of GENOPUS, 
 * A CNES Library of Flight Dynamics MMI in Java 
 * 
 * Copyright 2016-2026 CNES Centre National d’Etudes Spatiales 
 * Licensed to Centre National d’Etudes Spatiales (CNES) under one or more 
 * contributor license agreements. See the NOTICE file distributed with 
 * this work for additional information regarding copyright ownership. 
 * CNES licenses this file to You under the Apache License, Version 2.0 
 * (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.  
 * 
 **/

package observer;

import java.util.ArrayList;
import java.util.List;

import fr.cnes.genius.constraints.GConstraint;
import fr.cnes.genius.exception.GArithmeticException;
import fr.cnes.genius.exception.GComponentException;
import fr.cnes.genius.exception.GException;
import fr.cnes.genius.exception.GFileManipulatorException;
import fr.cnes.genius.exception.GRuntimeException;
import fr.cnes.genius.highLevel.GCheckBoxWithIndicator;
import fr.cnes.genius.highLevel.GEntryReal;
import fr.cnes.genius.highLevel.GLabelWithIndicator;
import fr.cnes.genius.interval.GIntervalAbstract.GStatus;
import fr.cnes.genius.logger.GLogger.Type;
import fr.cnes.genius.lowLevel.GLabel;
import fr.cnes.genius.lowLevel.GPanel;
import fr.cnes.genius.lowLevel.GSeparator;
import fr.cnes.genius.main.GClear;
import fr.cnes.genius.main.GComponent;
import fr.cnes.genius.main.GCondensedStatus;
import fr.cnes.genius.main.GCondensedStatusInterface;
import fr.cnes.genius.main.GDisplay;
import fr.cnes.genius.main.GEnvironment;
import fr.cnes.genius.main.GEvent;
import fr.cnes.genius.main.GListener;
import fr.cnes.genius.main.GReadWrite;
import fr.cnes.genius.observer.GObservable;
import fr.cnes.genius.observer.GObserver;
import fr.cnes.genius.observer.GObserverList;
import fr.cnes.genius.unit.GMetricUnit;
import fr.cnes.genius.unit.GUnit;
import fr.cnes.genopus.exception.GPAttitudeException;
import fr.cnes.genopus.exception.GPVehicleException;
import fr.cnes.genopus.main.GPItemType;
import fr.cnes.genopus.main.GPPatriusWithObjects;
import fr.cnes.genopus.maneuvers.GFuelTankEngineProvider;
import fr.cnes.genopus.maneuvers.GPEngine;
import fr.cnes.genopus.maneuvers.GPFuelTank;
import fr.cnes.genopus.vehicle.GPVehicleAeroProps;
import fr.cnes.genopus.vehicle.GPVehicleMassProps;
import fr.cnes.genopus.vehicle.GPVehiclePropulsiveProps;
import fr.cnes.genopus.vehicle.GPVehicleRadiativeProps;
import fr.cnes.genopus.vehicle.GPVehicleShape;
import fr.cnes.sirius.patrius.assembly.Assembly;
import fr.cnes.sirius.patrius.assembly.Vehicle;
import fr.cnes.sirius.patrius.assembly.properties.MassProperty;
import fr.cnes.sirius.patrius.assembly.properties.PropulsiveProperty;
import fr.cnes.sirius.patrius.assembly.properties.TankProperty;
import fr.cnes.sirius.patrius.assembly.properties.features.Facet;
import fr.cnes.sirius.patrius.assembly.vehicle.AerodynamicProperties;
import fr.cnes.sirius.patrius.assembly.vehicle.RadiativeProperties;
import fr.cnes.sirius.patrius.assembly.vehicle.VehicleSurfaceModel;
import fr.cnes.sirius.patrius.bodies.ExtendedOneAxisEllipsoid;
import fr.cnes.sirius.patrius.forces.atmospheres.Atmosphere;
import fr.cnes.sirius.patrius.frames.Frame;
import fr.cnes.sirius.patrius.math.geometry.euclidean.threed.CrossSectionProvider;
import fr.cnes.sirius.patrius.math.geometry.euclidean.threed.Sphere;
import fr.cnes.sirius.patrius.utils.exception.PatriusException;
import fr.cnes.sirius.patrius.utils.exception.PatriusRuntimeException;

/**
 * Widger for vehicle definition panel.
 * 
 * @genius.defaultConstraints 0:GLabelWithIndicator:label:0, 0 => "newline , gapleft 0 , split 2"<br>
 *                            0, 1 => "wmin 167"; 1:GEntryReal:totalMassEntryReal:1, 0, 0 =>
 *                            "newline , gapleft 20 , split 2"<br>
 *                            1, 0, 1 => (null)<br>
 *                            1, 1 => "split 2"<br>
 *                            1, 2 => (null); 2:GCheckBoxWithIndicator:massPropCheckbox:2, 0, 0 =>
 *                            "newline , gapleft 20 , split 2"<br>
 *                            2, 0, 1 => (null)<br>
 *                            2, 1 => (null); 3:GPVehicleMassProps:massProps:"newline"<br>
 *                            (see table below); 4:GPVehicleShape:vehicleShape:"newline"<br>
 *                            (see table below); 5:GCheckBoxWithIndicator:propulsivePropCheckbox:5,
 *                            0, 0 => "newline , gapleft 20 , split 2"<br>
 *                            5, 0, 1 => (null)<br>
 *                            5, 1 => (null);
 *                            6:GPVehiclePropulsiveProps:propulsiveProps:"newline , spanx 4 , growx"<br>
 *                            (see table below); 7:GCheckBoxWithIndicator:aeroPropCheckbox:7, 0, 0
 *                            => "newline , gapleft 20 , split 2"<br>
 *                            7, 0, 1 => (null)<br>
 *                            7, 1 => (null);
 *                            8:GPVehicleAeroProps:aeroProps:"newline , spanx 4 , growx"<br>
 *                            (see table below); 9:GCheckBoxWithIndicator:radiativePropCheckbox:9,
 *                            0, 0 => "newline , gapleft 20 , split 2"<br>
 *                            9, 0, 1 => (null)<br>
 *                            9, 1 => (null); 10:GPVehicleRadiativeProps:radiativeProps:"newline"<br>
 *                            (see table below)
 * @genius.defaultConstraints [[<i>massProps</i>]]3, 0:GEntryReal:dryMassEntryReal:3, 0, 0, 0 =>
 *                            "newline , gapleft 40 , split 2"<br>
 *                            3, 0, 0, 1 => (null)<br>
 *                            3, 0, 1 => "split 2"<br>
 *                            3, 0, 2 => (null)
 * @genius.defaultConstraints [[<i>vehicleShape</i>]]4, 0:GLabelWithIndicator:label:4, 0, 0 =>
 *                            "newline , gapleft 20 , split 2"<br>
 *                            4, 0, 1 => (null); 4, 1:GChoice:shapeChoice:4, 1, 0, 0 =>
 *                            "newline , gapleft 40 , split 2"<br>
 *                            4, 1, 0, 1 => (null)<br>
 *                            4, 1, 1 => "split"; 4, 2:GChoice:shapeDefChoice:4, 2, 0, 0 =>
 *                            "newline , gapleft 40 , split 2"<br>
 *                            4, 2, 0, 1 => (null)<br>
 *                            4, 2, 1 => "split"; 4, 3:GEntryReal:sphRadEntryReal:4, 3, 0, 0 =>
 *                            "newline , gapleft 60 , split 2"<br>
 *                            4, 3, 0, 1 => (null)<br>
 *                            4, 3, 1 => "split 2"<br>
 *                            4, 3, 2 => (null); 4, 4:GEntryReal:sphSurfEntryReal:4, 4, 0, 0 =>
 *                            "newline , gapleft 60 , split 2"<br>
 *                            4, 4, 0, 1 => (null)<br>
 *                            4, 4, 1 => "split 2"<br>
 *                            4, 4, 2 => (null); 4, 5:GEntryReal:parallLengthEntryReal:4, 5, 0, 0 =>
 *                            "newline , gapleft 60 , split 2"<br>
 *                            4, 5, 0, 1 => (null)<br>
 *                            4, 5, 1 => "split 2"<br>
 *                            4, 5, 2 => (null); 4, 6:GEntryReal:parallWidthEntryReal:4, 6, 0, 0 =>
 *                            "newline , gapleft 60 , split 2"<br>
 *                            4, 6, 0, 1 => (null)<br>
 *                            4, 6, 1 => "split 2"<br>
 *                            4, 6, 2 => (null); 4, 7:GEntryReal:parallHeightEntryReal:4, 7, 0, 0 =>
 *                            "newline , gapleft 60 , split 2"<br>
 *                            4, 7, 0, 1 => (null)<br>
 *                            4, 7, 1 => "split 2"<br>
 *                            4, 7, 2 => (null); 4, 8:GEntryReal:parallXSurfEntryReal:4, 8, 0, 0 =>
 *                            "newline , gapleft 60 , split 2"<br>
 *                            4, 8, 0, 1 => (null)<br>
 *                            4, 8, 1 => "split 2"<br>
 *                            4, 8, 2 => (null); 4, 9:GEntryReal:parallYSurfEntryReal:4, 9, 0, 0 =>
 *                            "newline , gapleft 60 , split 2"<br>
 *                            4, 9, 0, 1 => (null)<br>
 *                            4, 9, 1 => "split 2"<br>
 *                            4, 9, 2 => (null); 4, 10:GEntryReal:parallZSurfEntryReal:4, 10, 0, 0
 *                            => "newline , gapleft 60 , split 2"<br>
 *                            4, 10, 0, 1 => (null)<br>
 *                            4, 10, 1 => "split 2"<br>
 *                            4, 10, 2 => (null); 4, 11:GEntryReal:cylRadEntryReal:4, 11, 0, 0 =>
 *                            "newline , gapleft 60 , split 2"<br>
 *                            4, 11, 0, 1 => (null)<br>
 *                            4, 11, 1 => "split 2"<br>
 *                            4, 11, 2 => (null); 4, 12:GEntryReal:cylLengthEntryReal:4, 12, 0, 0 =>
 *                            "newline , gapleft 60 , split 2"<br>
 *                            4, 12, 0, 1 => (null)<br>
 *                            4, 12, 1 => "split 2"<br>
 *                            4, 12, 2 => (null); 4, 13:GEntryReal:cylXSurfEntryReal:4, 13, 0, 0 =>
 *                            "newline , gapleft 60 , split 2"<br>
 *                            4, 13, 0, 1 => (null)<br>
 *                            4, 13, 1 => "split 2"<br>
 *                            4, 13, 2 => (null); 4, 14:GEntryReal:cylTSurfEntryReal:4, 14, 0, 0 =>
 *                            "newline , gapleft 60 , split 2"<br>
 *                            4, 14, 0, 1 => (null)<br>
 *                            4, 14, 1 => "split 2"<br>
 *                            4, 14, 2 => (null); 4, 15:GCheckBoxWithIndicator:solarPanelsChoice:4,
 *                            15, 0, 0 => "newline , gapleft 40 , split 2"<br>
 *                            4, 15, 0, 1 => (null)<br>
 *                            4, 15, 1 => (null); 4, 16:GEntryReal:panelXSurfEntryReal:4, 16, 0, 0
 *                            => "newline , gapleft 60 , split 2"<br>
 *                            4, 16, 0, 1 => (null)<br>
 *                            4, 16, 1 => "split 2"<br>
 *                            4, 16, 2 => (null); 4, 17:GEntryReal:panelYSurfEntryReal:4, 17, 0, 0
 *                            => "newline , gapleft 60 , split 2"<br>
 *                            4, 17, 0, 1 => (null)<br>
 *                            4, 17, 1 => "split 2"<br>
 *                            4, 17, 2 => (null); 4, 18:GEntryReal:panelZSurfEntryReal:4, 18, 0, 0
 *                            => "newline , gapleft 60 , split 2"<br>
 *                            4, 18, 0, 1 => (null)<br>
 *                            4, 18, 1 => "split 2"<br>
 *                            4, 18, 2 => (null)
 * @genius.defaultConstraints [[<i>propulsiveProps</i>]]6, 0:GEntryReal:ergMassEntryReal:6, 0, 0, 0
 *                            => "newline , gapleft 40 , split 2"<br>
 *                            6, 0, 0, 1 => (null)<br>
 *                            6, 0, 1 => (null)<br>
 *                            6, 0, 2 => (null); 6,
 *                            1:GSeparator:enginesSeparator:"newline , gapleft 40 , spanx 4 , growx"
 *                            ; 6,
 *                            2:GOpenCloseButton:enginesOpenCloseButton:"newline , gapleft 20 , split 2"
 *                            ; 6, 3:GLabelWithIndicator:enginesLabel:6, 3, 0 => (null)<br>
 *                            6, 3, 1 => (null); 6,
 *                            4:GComponentList:enginesList:"newline , gapleft 40 , spanx 4 , growx"<br>
 *                            (see table below); 6,
 *                            5:GSeparator:tanksSeparator:"newline , gapleft 40 , spanx 4 , growx";
 *                            6,
 *                            6:GOpenCloseButton:tanksOpenCloseButton:"newline , gapleft 20 , split 2"
 *                            ; 6, 7:GLabelWithIndicator:tanksLabel:6, 7, 0 => (null)<br>
 *                            6, 7, 1 => (null); 6,
 *                            8:GComponentList:tanksList:"newline , gapleft 40 , spanx 4 , growx"<br>
 *                            (see table below)
 * @genius.defaultConstraints [[<i>enginesList</i>]]6, 4, 0:GHeaderPanel:header:"hmin 40"<br>
 *                            (see table below); 6, 4, 1:GLabel:spaceLabel:"newline , height 0"
 * @genius.defaultConstraints [[<i>header</i>]]6, 4, 0, 0:GEntryInt:totalNumberItemsEntry:6, 4, 0,
 *                            0, 0, 0 => "newline , split 2"<br>
 *                            6, 4, 0, 0, 0, 1 => (null)<br>
 *                            6, 4, 0, 0, 1 => "skip 2 , width 50"; 6, 4, 0,
 *                            1:GEntryInt:currentItemRankEntry:6, 4, 0, 1, 0, 0 => "split 2"<br>
 *                            6, 4, 0, 1, 0, 1 => (null)<br>
 *                            6, 4, 0, 1, 1 => "width 50"; 6, 4, 0,
 *                            2:GButton:decrButton:"alignx Right , height 20"; 6, 4, 0,
 *                            3:GButton:incrButton:"height 20"; 6, 4, 0, 4:GPopupList:menu:(null)
 * @genius.defaultConstraints [[<i>tanksList</i>]]6, 8, 0:GHeaderPanel:header:"hmin 40"<br>
 *                            (see table below); 6, 8, 1:GLabel:spaceLabel:"newline , height 0"
 * @genius.defaultConstraints [[<i>header</i>]]6, 8, 0, 0:GEntryInt:totalNumberItemsEntry:6, 8, 0,
 *                            0, 0, 0 => "newline , split 2"<br>
 *                            6, 8, 0, 0, 0, 1 => (null)<br>
 *                            6, 8, 0, 0, 1 => "skip 2 , width 50"; 6, 8, 0,
 *                            1:GEntryInt:currentItemRankEntry:6, 8, 0, 1, 0, 0 => "split 2"<br>
 *                            6, 8, 0, 1, 0, 1 => (null)<br>
 *                            6, 8, 0, 1, 1 => "width 50"; 6, 8, 0,
 *                            2:GButton:decrButton:"alignx Right , height 20"; 6, 8, 0,
 *                            3:GButton:incrButton:"height 20"; 6, 8, 0, 4:GPopupList:menu:(null)
 * @genius.defaultConstraints [[<i>aeroProps</i>]]8, 0:GChoice:coeffsTypeChoice:8, 0, 0, 0 =>
 *                            "newline , gapleft 40 , split 2"<br>
 *                            8, 0, 0, 1 => (null)<br>
 *                            8, 0, 1 => "split"; 8, 1:GEntryReal:dragCoeffEntryReal:8, 1, 0, 0 =>
 *                            "newline , gapleft 40 , split 2"<br>
 *                            8, 1, 0, 1 => (null)<br>
 *                            8, 1, 1 => (null)<br>
 *                            ; 8, 2:GEntryReal:liftCoeffEntryReal:8, 2, 0, 0 =>
 *                            "newline , gapleft 40 , split 2"<br>
 *                            8, 2, 0, 1 => (null)<br>
 *                            8, 2, 1 => (null)<br>
 *                            ; 8, 3:GChoice:variableChoice:8, 3, 0, 0 =>
 *                            "newline , gapleft 40 , split 2"<br>
 *                            8, 3, 0, 1 => (null)<br>
 *                            8, 3, 1 => "split"; 8,
 *                            4:GPAeroCoeffByOneVar:altitudeTable1D:"newline , gapleft 40 , spanx 3"<br>
 *                            (see table below); 8,
 *                            5:GPAeroCoeffByOneVar:aoATable1D:"newline , gapleft 40 , spanx 3"<br>
 *                            (see table below); 8,
 *                            6:GPAeroCoeffByOneVar:machTable1D:"newline , gapleft 40 , spanx 3"<br>
 *                            (see table below); 8,
 *                            7:GPAeroCoeffByTwoVar:aoAandMachTable2D:"newline , gapleft 40 , spanx 3"
 * <br>
 *                            (see table below)
 * @genius.defaultConstraints [[<i>altitudeTable1D</i>]]8, 4, 0:GLabelWithIndicator:title:8, 4, 0, 0
 *                            => "newline , split 2"<br>
 *                            8, 4, 0, 1 => "wrap"; 8, 4,
 *                            1:GEntryRealVector:altitudeRealVec:"newline"<br>
 *                            (see table below); 8, 4, 2:GEntryRealVector:dragRealVec:"newline"<br>
 *                            (see table below); 8, 4, 3:GEntryRealVector:liftRealVec:"newline"<br>
 *                            (see table below); 8, 4, 4:GPopupList:popup_0:(null); 8, 4,
 *                            5:GPopupList:popup_1:(null)
 * @genius.defaultConstraints [[<i>altitudeRealVec</i>]]8, 4, 1, 0:GLabelWithIndicator:label:8, 4,
 *                            1, 0, 0 => "split 2 , alignx Center"<br>
 *                            8, 4, 1, 0, 1 => (null); 8, 4, 1, 1:GRealField:field_1:"newline"; 8,
 *                            4, 1, 2:GPopupList:unitPopup:"alignx Center , cell 0 2"
 * @genius.defaultConstraints [[<i>dragRealVec</i>]]8, 4, 2, 0:GLabelWithIndicator:label:8, 4, 2, 0,
 *                            0 => "split 2 , alignx Center"<br>
 *                            8, 4, 2, 0, 1 => (null); 8, 4, 2, 1:GRealField:field_1:""
 * @genius.defaultConstraints [[<i>liftRealVec</i>]]8, 4, 3, 0:GLabelWithIndicator:label:8, 4, 3, 0,
 *                            0 => "split 2 , alignx Center"<br>
 *                            8, 4, 3, 0, 1 => (null); 8, 4, 3, 1:GRealField:field_1:""
 * @genius.defaultConstraints [[<i>aoATable1D</i>]]8, 5, 0:GLabelWithIndicator:title:8, 5, 0, 0 =>
 *                            "newline , split 2"<br>
 *                            8, 5, 0, 1 => "wrap"; 8, 5, 1:GEntryRealVector:aoARealVec:"newline"<br>
 *                            (see table below); 8, 5, 2:GEntryRealVector:dragRealVec:"newline"<br>
 *                            (see table below); 8, 5, 3:GEntryRealVector:liftRealVec:"newline"<br>
 *                            (see table below); 8, 5, 4:GPopupList:popup_0:(null); 8, 5,
 *                            5:GPopupList:popup_1:(null)
 * @genius.defaultConstraints [[<i>aoARealVec</i>]]8, 5, 1, 0:GLabelWithIndicator:label:8, 5, 1, 0,
 *                            0 => "split 2 , alignx Center"<br>
 *                            8, 5, 1, 0, 1 => (null); 8, 5, 1, 1:GRealField:field_1:"newline"; 8,
 *                            5, 1, 2:GPopupList:unitPopup:"alignx Center , cell 0 2"
 * @genius.defaultConstraints [[<i>dragRealVec</i>]]8, 5, 2, 0:GLabelWithIndicator:label:8, 5, 2, 0,
 *                            0 => "split 2 , alignx Center"<br>
 *                            8, 5, 2, 0, 1 => (null); 8, 5, 2, 1:GRealField:field_1:""
 * @genius.defaultConstraints [[<i>liftRealVec</i>]]8, 5, 3, 0:GLabelWithIndicator:label:8, 5, 3, 0,
 *                            0 => "split 2 , alignx Center"<br>
 *                            8, 5, 3, 0, 1 => (null); 8, 5, 3, 1:GRealField:field_1:""
 * @genius.defaultConstraints [[<i>machTable1D</i>]]8, 6, 0:GLabelWithIndicator:title:8, 6, 0, 0 =>
 *                            "newline , split 2"<br>
 *                            8, 6, 0, 1 => "wrap"; 8, 6, 1:GEntryRealVector:machRealVec:"newline"<br>
 *                            (see table below); 8, 6, 2:GEntryRealVector:dragRealVec:"newline"<br>
 *                            (see table below); 8, 6, 3:GEntryRealVector:liftRealVec:"newline"<br>
 *                            (see table below); 8, 6, 4:GPopupList:popup_0:(null); 8, 6,
 *                            5:GPopupList:popup_1:(null)
 * @genius.defaultConstraints [[<i>machRealVec</i>]]8, 6, 1, 0:GLabelWithIndicator:label:8, 6, 1, 0,
 *                            0 => "split 2 , alignx Center"<br>
 *                            8, 6, 1, 0, 1 => (null); 8, 6, 1, 1:GRealField:field_1:"newline"
 * @genius.defaultConstraints [[<i>dragRealVec</i>]]8, 6, 2, 0:GLabelWithIndicator:label:8, 6, 2, 0,
 *                            0 => "split 2 , alignx Center"<br>
 *                            8, 6, 2, 0, 1 => (null); 8, 6, 2, 1:GRealField:field_1:""
 * @genius.defaultConstraints [[<i>liftRealVec</i>]]8, 6, 3, 0:GLabelWithIndicator:label:8, 6, 3, 0,
 *                            0 => "split 2 , alignx Center"<br>
 *                            8, 6, 3, 0, 1 => (null); 8, 6, 3, 1:GRealField:field_1:""
 * @genius.defaultConstraints [[<i>aoAandMachTable2D</i>]]8, 7, 0:GChoice:aoAandMachCoeffChoice:8,
 *                            7, 0, 0, 0 => "newline , gapleft 0 , split 2"<br>
 *                            8, 7, 0, 0, 1 => (null)<br>
 *                            8, 7, 0, 1 => "split"; 8, 7,
 *                            1:GTable2D:aoAandMachCxTable2D:"newline , gapleft 0 , spanx 3"<br>
 *                            (see table below); 8, 7,
 *                            2:GTable2D:aoAandMachCzTable2D:"newline , gapleft 0 , spanx 3"<br>
 *                            (see table below)
 * @genius.defaultConstraints [[<i>aoAandMachCxTable2D</i>]]8, 7, 1,
 *                            0:GLabel:dataLabel:"newline , spanx 2 , alignx Center"; 8, 7, 1,
 *                            1:GLabel
 *                            :columnHeaderLabel:"newline , spanx 2 , split 2 , alignx Center"; 8,
 *                            7, 1, 2:GLabel:rowHeaderLabel:"newline , aligny Center"; 8, 7, 1,
 *                            3:GTable:table:"wmax 450 , hmax 200"
 * @genius.defaultConstraints [[<i>aoAandMachCzTable2D</i>]]8, 7, 2,
 *                            0:GLabel:dataLabel:"newline , spanx 2 , alignx Center"; 8, 7, 2,
 *                            1:GLabel
 *                            :columnHeaderLabel:"newline , spanx 2 , split 2 , alignx Center"; 8,
 *                            7, 2, 2:GLabel:rowHeaderLabel:"newline , aligny Center"; 8, 7, 2,
 *                            3:GTable:table:"wmax 450 , hmax 200"
 * @genius.defaultConstraints [[<i>radiativeProps</i>]]10,
 *                            0:GPRadiativeProperty:radiativeProperty:"newline"<br>
 *                            (see table below); 10, 1:GCheckBoxWithIndicator:infraPropCheckbox:10,
 *                            1, 0, 0 => "newline , gapleft 40 , split 2"<br>
 *                            10, 1, 0, 1 => (null)<br>
 *                            10, 1, 1 => (null); 10,
 *                            2:GPRadiativeIRProperty:radiativeIRProperty:"newline"<br>
 *                            (see table below)
 * @genius.defaultConstraints [[<i>radiativeProperty</i>]]10, 0, 0:GLabelWithIndicator:label:10, 0,
 *                            0, 0 => "newline , gapleft 40 , split 2"<br>
 *                            10, 0, 0, 1 => (null); 10, 0, 1:GEntryReal:diffuseCoefEntryReal:10, 0,
 *                            1, 0, 0 => "newline , gapleft 60 , split 2"<br>
 *                            10, 0, 1, 0, 1 => (null)<br>
 *                            10, 0, 1, 1 => (null)<br>
 *                            ; 10, 0, 2:GEntryReal:specularCoefEntryReal:10, 0, 2, 0, 0 =>
 *                            "newline , gapleft 60 , split 2"<br>
 *                            10, 0, 2, 0, 1 => (null)<br>
 *                            10, 0, 2, 1 => (null)<br>
 *                            ; 10, 0, 3:GEntryReal:diffuseCoefEntryReal:10, 0, 3, 0, 0 =>
 *                            "newline , gapleft 60 , split 2"<br>
 *                            10, 0, 3, 0, 1 => (null)<br>
 *                            10, 0, 3, 1 => (null)<br>
 * @genius.defaultConstraints [[<i>radiativeIRProperty</i>]]10, 2, 0:GLabelWithIndicator:label:10,
 *                            2, 0, 0 => "newline , gapleft 60 , split 2"<br>
 *                            10, 2, 0, 1 => (null); 10, 2, 1:GEntryReal:diffuseCoefEntryReal:10, 2,
 *                            1, 0, 0 => "newline , gapleft 80 , split 2"<br>
 *                            10, 2, 1, 0, 1 => (null)<br>
 *                            10, 2, 1, 1 => (null)<br>
 *                            ; 10, 2, 2:GEntryReal:specularCoefEntryReal:10, 2, 2, 0, 0 =>
 *                            "newline , gapleft 80 , split 2"<br>
 *                            10, 2, 2, 0, 1 => (null)<br>
 *                            10, 2, 2, 1 => (null)<br>
 *                            ; 10, 2, 3:GEntryReal:diffuseCoefEntryReal:10, 2, 3, 0, 0 =>
 *                            "newline , gapleft 80 , split 2"<br>
 *                            10, 2, 3, 0, 1 => (null)<br>
 *                            10, 2, 3, 1 => (null)<br>
 * 
 * @author ffsm
 * 
 */
public class GPVehicleLocal extends GPanel implements GReadWrite, GListener, GClear, GDisplay,
        GCondensedStatusInterface, GObservable, GFuelTankEngineProvider,
        GPPatriusWithObjects<Vehicle>, GObserver {

    // MSG_STATEMENT
    private static final String MISSING_PROPERTY_ERROR = GEnvironment
            .marktr("Vehicle has no %s properties.");
    // MSG_STATEMENT
    private static final String INCOMPAT_WITH_WIDGET_ERROR = GEnvironment
            .marktr("Input vehicle has %s but the ihm has not");
    private static final String PROPULSIVE_STR = GEnvironment.marktr("propulsive");

    /** Message. */
    private static final String INCOMPATIBLE_AERO_PROPERTIES = GEnvironment
            .marktr("Drag and Lift Aerodynamic coefficients are not of the same type");

    /** Boolean to show masses. */
    private final boolean m_hasMassProps;
    /** Boolean to show engines and tanks definition. */
    private final boolean m_hasPropulsiveProps;
    /** Boolean to show aerodynamic properties. */
    private final boolean m_hasAeroProps;
    /** Boolean to show radiative properties. */
    private final boolean m_hasRadiativeProps;

    /** Label. */
    private final GLabelWithIndicator m_label;

    /** Total mass. */
    private final GEntryReal m_totalMassEntryReal;

    /** Mass. */
    private GPVehicleMassProps m_massProps = null;

    /** Flag to show or not the Mass properties and the total mass. */
    private boolean m_massPropsVisible;

    /** Vehicle shape. */
    private final GPVehicleShape m_vehicleShape;

    /** Propulsive properties checkbox. */
    private GCheckBoxWithIndicator m_propulsivePropCheckbox = null;

    /** Tanks and engines. */
    private GPVehiclePropulsiveProps m_propulsiveProps = null;

    /** Aero properties checkbox. */
    private GCheckBoxWithIndicator m_aeroPropCheckbox = null;

    /** Expert sphere coefficients. */
    private GPVehicleAeroProps m_aeroProps = null;

    /** Radiative properties checkbox. */
    private GCheckBoxWithIndicator m_radiativePropCheckbox = null;

    /** Radiative properties. */
    private GPVehicleRadiativeProps m_radiativeProps = null;

    /** Separators. */
    private final GSeparator[] m_separators = new GSeparator[5];

    /** Update management. */
    private final GObserverList m_observerList;

    /**
     * Create a new instance with empty label. The widget id will be assigned
     * automatically.
     * 
     */
    public GPVehicleLocal() {
        this("");
    }
    
    /**
     * Create a new instance with all properties active. The widget id will be assigned
     * automatically. The Earth's shape must be initialized for aerodynamic properties. The
     * atmosphere is initialized to a SimpleExponentialAtmosphere with rho0 = 0.0004, h0 = 42000.0
     * and hscale = 7500.0.
     * 
     * @param label widget label
     */
    public GPVehicleLocal(final String label) {
        this(null, label, true, true, true, true, true);
    }

    /**
     * Create a new instance with all properties active. The widget id will be assigned
     * automatically. The Earth's shape must be initialized for aerodynamic properties. The
     * atmosphere is initialized to a SimpleExponentialAtmosphere with rho0 = 0.0004, h0 = 42000.0
     * and hscale = 7500.0.
     * 
     * @param widgetId widget identifier, if null an identifier will be automatically created
     * @param label widget label
     */
    public GPVehicleLocal(final String widgetId, final String label) {
        this(widgetId, label, true, true, true, true, true);
    }

    /**
     * Create a new instance. The widget id will be assigned automatically.
     * 
     * @param label widget label
     * @param showMassProperties if true, masses appear (dry and total)
     * @param showPropulsiveProperties if true, propulsive properties appear
     * @param showAeroProperties if true, aerodynamic properties appear
     * @param showAeroExpertSettings if true and showAeroProperties true, tabulated coefficients
     *        appear too for sphere shape
     * @param showRadiativeProperties if true, radiative properties appear
     */
    public GPVehicleLocal(final String label, final boolean showMassProperties, final boolean showPropulsiveProperties,
            final boolean showAeroProperties, final boolean showAeroExpertSettings,
            final boolean showRadiativeProperties) {
        this(null, label, showMassProperties, showPropulsiveProperties, showAeroProperties,
                showAeroExpertSettings, showRadiativeProperties);
    }

    /**
     * Create a new instance.
     * 
     * @param widgetId widget identifier, if null an identifier will be automatically created
     * @param label widget label
     * @param showMassProperties if true, masses appear (dry and total)
     * @param showPropulsiveProperties if true, propulsive properties appear
     * @param showAeroProperties if true, aerodynamic properties appear
     * @param showAeroExpertSettings if true and showAeroProperties true, tabulated coefficients
     *        appear too for sphere shape
     * @param showRadiativeProperties if true, radiative properties appear
     */
    public GPVehicleLocal(final String widgetId, final String label, final boolean showMassProperties,
            final boolean showPropulsiveProperties, final boolean showAeroProperties,
            final boolean showAeroExpertSettings, final boolean showRadiativeProperties) {
        super(widgetId);
        setNameInConfigFile(label);

        m_hasMassProps = showMassProperties;
        m_hasPropulsiveProps = showPropulsiveProperties;
        m_hasAeroProps = showAeroProperties;
        m_hasRadiativeProps = showRadiativeProperties;

        m_label = new GLabelWithIndicator(label);

        m_totalMassEntryReal = new GEntryReal(GEnvironment.tr("Total mass:"), 0.0, new GUnit[] {
            new GMetricUnit("kg"), new GMetricUnit("g") });
        m_totalMassEntryReal.setNameInConfigFile("totalMass");
        m_totalMassEntryReal.setCommentInConfigFile(null);
        m_totalMassEntryReal.setDisplayIsModifiedIndicator(DisplayIndicatorStatus.Never);
        m_totalMassEntryReal.setEnabled(false);

        m_massPropsVisible = true;

        if (m_hasMassProps) {
            m_massProps = new GPVehicleMassProps(null, "massProperties");
        }

        m_vehicleShape = new GPVehicleShape(null, GEnvironment.tr("Shape"));
        m_vehicleShape.setNameInConfigFile("VehicleShape");
        m_vehicleShape.setCommentInConfigFile(null);

        if (m_hasPropulsiveProps) {
            m_propulsivePropCheckbox = new GCheckBoxWithIndicator(
                    GEnvironment.tr("Propulsive properties"), false);
            m_propulsivePropCheckbox.setNameInConfigFile("hasPropulsiveProperties");
            m_propulsivePropCheckbox.setCommentInConfigFile(null);

            m_propulsiveProps = new GPVehiclePropulsiveProps(null, "PropulsiveProperties");

        }

        if (m_hasAeroProps) {
            m_aeroPropCheckbox = new GCheckBoxWithIndicator(
                    GEnvironment.tr("Aerodynamic properties"), false);
            m_aeroPropCheckbox.setNameInConfigFile("hasAerodynamicProperties");
            m_aeroPropCheckbox.setCommentInConfigFile(null);

            m_aeroProps = new GPVehicleAeroProps(null, "AerodynamicProperties",
                    showAeroExpertSettings);
        }

        if (m_hasRadiativeProps) {
            m_radiativePropCheckbox = new GCheckBoxWithIndicator(
                    GEnvironment.tr("Radiative properties"), false);
            m_radiativePropCheckbox.setNameInConfigFile("hasRadiativeProperties");
            m_radiativePropCheckbox.setCommentInConfigFile(null);

            m_radiativeProps = new GPVehicleRadiativeProps(null, "RadiativeProperties");
        }

        for (int i = 0; i < m_separators.length; i++) {
            m_separators[i] = new GSeparator();
        }

        m_observerList = new GObserverList();

        updateTotalMass();

        // Updating status
        updateStatus();

        setDefaultConstraints();

        // Setting which is the class and the rank applying for constraints by
        // type
        activateConstraintsByType(getClass());

        setWidgetId(widgetId);

        attachCopyPasteMenu();
    }

    /**
     * Create a new instance with all properties active and with an initial vehicle. The widget id
     * will be assigned automatically. The Earth's shape must be initialized for aerodynamic
     * properties. The atmosphere is initialized to a SimpleExponentialAtmosphere with rho0 =
     * 0.0004, h0 = 42000.0 and hscale = 7500.0.
     * 
     * @param label widget label
     * @param vehicle initial vehicle to be set in the widget.
     */
    public GPVehicleLocal(final String label, final Vehicle vehicle) {
        this(null, label, vehicle, true, true, true, true, true);
    }

    /**
     * Create a new instance with an initial vehicle. The widget id will be assigned automatically.
     * 
     * @param label widget label
     * @param vehicle initial vehicle to be set in the widget. IMPORTANT: It must be compatible with
     *        the different show*Properties arguments.
     * @param showMassProperties if true, masses appear (dry and total)
     * @param showPropulsiveProperties if true, propulsive properties appear
     * @param showAeroProperties if true, aerodynamic properties appear
     * @param showAeroExpertSettings if true and showAeroProperties true, tabulated coefficients
     *        appear too for sphere shape
     * @param showRadiativeProperties if true, radiative properties appear
     */
    public GPVehicleLocal(final String label, final Vehicle vehicle, final boolean showMassProperties,
            final boolean showPropulsiveProperties, final boolean showAeroProperties,
            final boolean showAeroExpertSettings, final boolean showRadiativeProperties) {
        this(null, label, vehicle, showMassProperties, showPropulsiveProperties,
                showAeroProperties, showAeroExpertSettings, showRadiativeProperties);
    }

    /**
     * Create a new instance with an initial vehicle.
     * 
     * @param widgetId widget identifier, if null an identifier will be automatically created
     * @param label widget label
     * @param vehicle initial vehicle to be set in the widget. IMPORTANT: It must be compatible with
     *        the different show*Properties arguments.
     * @param showMassProperties if true, masses appear (dry and total)
     * @param showPropulsiveProperties if true, propulsive properties appear
     * @param showAeroProperties if true, aerodynamic properties appear
     * @param showAeroExpertSettings if true and showAeroProperties true, tabulated coefficients
     *        appear too for sphere shape
     * @param showRadiativeProperties if true, radiative properties appear
     */

    public GPVehicleLocal(final String widgetId, final String label, final Vehicle vehicle, final boolean showMassProperties,
            final boolean showPropulsiveProperties, final boolean showAeroProperties,
            final boolean showAeroExpertSettings, final boolean showRadiativeProperties) {
        this(widgetId, label, showMassProperties, showPropulsiveProperties, showAeroProperties,
                showAeroExpertSettings, showRadiativeProperties);

        if (showPropulsiveProperties) {
            GPEngine.setCount(vehicle.getEngineCount());
            GPFuelTank.setCount(vehicle.getTankCount());
        }
        if (vehicle != null) {
            try {
                setVehicle(vehicle);
                setDefaultVehicle(vehicle);
                setSavedVehicle(vehicle);
            } catch (final GPVehicleException e) {
                throw new GRuntimeException(e);
            } catch (final GArithmeticException e) {
                // if dryMass is NaN, or if vehicle shape has sphere, parallelepiped, cylinder or
                // solar panels
                // surfaces equal to NaN, or if aero coeffs are constant and NaN , or if any of
                // the radiative coefficients is NaN.
                throw new GRuntimeException(e);
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setWidgetId(final String widgetId) {
        super.setWidgetId(widgetId);

        m_label.setWidgetId(composeWidgetId("label"));
        m_totalMassEntryReal.setWidgetId(composeWidgetId("totalMassEntryReal"));
        if (m_hasMassProps) {
            m_massProps.setWidgetId(composeWidgetId("massProps"));
        }
        m_vehicleShape.setWidgetId(composeWidgetId("vehicleShape"));
        if (m_hasPropulsiveProps) {
            m_propulsivePropCheckbox.setWidgetId(composeWidgetId("propulsivePropCheckbox"));
            m_propulsiveProps.setWidgetId(composeWidgetId("propulsiveProps"));
        }
        if (m_hasAeroProps) {
            m_aeroPropCheckbox.setWidgetId(composeWidgetId("aeroPropCheckbox"));
            m_aeroProps.setWidgetId(composeWidgetId("aeroProps"));
        }
        if (m_hasRadiativeProps) {
            m_radiativePropCheckbox.setWidgetId(composeWidgetId("radiativePropCheckbox"));
            m_radiativeProps.setWidgetId(composeWidgetId("radiativeProps"));
        }
        for (int i = 0; i < m_separators.length; i++) {
            m_separators[i].setWidgetId(composeWidgetId("separator_" + i));
        }
    }

    /**
     * Setter for the default constraint of the widget.
     */
    @Override
    public final void setDefaultConstraints() {
        setConstraint(0);
    }

    /**
     * Setter for the constraints of the widget.
     * 
     * @param widgetGap left gap from panel border to display the label of the widget.
     */
    public void setConstraint(final int widgetGap) {
        final int SUB_GAP = 20;

        final GConstraint firstLevelConstraint = new GConstraint(GConstraint.newline(),
                GConstraint.gapleft(widgetGap), GConstraint.split(2));

        m_label.setInnerDescendantConstraint(firstLevelConstraint, 0);

        if (m_hasRadiativeProps) {
            // This is to avoid horizontal displacement
            // The longest label text : "Absorption Coefficient:"
            final GLabel longestGLabel = m_radiativeProps.getGPRadiativeIRProperty()
                    .getGEntryRealAbsCoeff().getGLabelWithIndicator().getGLabel();
            // The always visible label : the widget's label
            final GLabelWithIndicator alwaysVisibleGLabelWithIndicator = m_label;
            // Avoid horizontal displacement:
            GConstraint
                    .avoidHorizontalDisplacement(longestGLabel, alwaysVisibleGLabelWithIndicator);
        }

        final GConstraint secondLevelConstraint = new GConstraint(GConstraint.newline(),
                GConstraint.gapleft(widgetGap + SUB_GAP), GConstraint.split(2));

        m_totalMassEntryReal.setInnerDescendantConstraint(secondLevelConstraint, 0, 0);

        final GConstraint fieldsConstraint = new GConstraint(GConstraint.split(2));
        m_totalMassEntryReal.setInnerDescendantConstraint(fieldsConstraint, 1);

        if (m_hasMassProps) {
            m_massProps.setConstraint(widgetGap);
        }
        m_vehicleShape.setConstraint(widgetGap + SUB_GAP);
        if (m_hasPropulsiveProps) {
            m_propulsivePropCheckbox.setInnerDescendantConstraint(secondLevelConstraint, 0, 0);
            // It is a GPanel:
            m_propulsiveProps.setConstraint(new GConstraint(GConstraint.newline(), GConstraint
                    .spanx(4), GConstraint.growx()));
            // Constraints for its subcomponents:
            m_propulsiveProps.setConstraint(widgetGap + SUB_GAP);
        }
        if (m_hasAeroProps) {
            m_aeroPropCheckbox.setInnerDescendantConstraint(secondLevelConstraint, 0, 0);
            // It is a GPanel:
            m_aeroProps.setConstraint(new GConstraint(GConstraint.newline(), GConstraint.spanx(4),
                    GConstraint.growx()));
            // Constraints for its subcomponents:
            m_aeroProps.setConstraint(widgetGap + SUB_GAP);
        }
        if (m_hasRadiativeProps) {
            m_radiativePropCheckbox.setInnerDescendantConstraint(secondLevelConstraint, 0, 0);
            m_radiativeProps.setConstraint(widgetGap + SUB_GAP);
        }
        for (int i = 0; i < m_separators.length; i++) {
            m_separators[i].setConstraint(new GConstraint(GConstraint.newline(), GConstraint
                    .spanx(4), GConstraint.growx()));
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void attachCopyPasteMenu(final String label, final CopyPasteOperation... operationList) {
        m_label.attachCopyPasteMenu(label, operationList);
    }

    /**
     * Getter for the label of the widget.
     * 
     * @return the label of the widget.
     */
    public GLabelWithIndicator getGLabel() {
        return m_label;
    }

    /**
     * Getter for the entry real for the total mass.
     * 
     * @return the entry real for the total mass.
     */
    public GEntryReal getGEntryRealTotalMass() {
        return m_totalMassEntryReal;
    }

    /**
     * Getter for the separator before the mass properties sub widget.
     * 
     * @return the separator before the mass properties sub widget
     */
    public GComponent getGSeparatorMassProps() {
        return m_separators[0];
    }

    /**
     * Getter for the entry real to set the dry mass.
     * 
     * @return the entry real to set the dry mass.
     */
    public GPVehicleMassProps getGPVehicleMassProps() {
        return m_massProps;
    }

    /**
     * Getter for the separator before the vehicle shape sub widget.
     * 
     * @return the separator before the vehicle shape sub widget
     */
    public GComponent getGSeparatorVehicleShape() {
        return m_separators[1];
    }

    /**
     * Getter for the widget to set the vehicle shape.
     * 
     * @return the widget to set the vehicle shape.
     */
    public GPVehicleShape getGPVehicleShape() {
        return m_vehicleShape;
    }

    /**
     * Getter for the separator before the propulsive properties sub widget.
     * 
     * @return the separator before the propulsive properties sub widget
     */
    public GComponent getGSeparatorPropulsiveProps() {
        return m_separators[2];
    }

    /**
     * Getter for the checkbox for the propulsive properties visibility.
     * 
     * @return the checkbox for the propulsive properties visibility.
     */
    public GCheckBoxWithIndicator getGCheckBoxPropulsiveProps() {
        return m_propulsivePropCheckbox;
    }

    /**
     * Getter for the widget to set the vehicle propulsive properties.
     * 
     * @return the widget to set the vehicle propulsive properties.
     */
    public GPVehiclePropulsiveProps getGPVehiclePropulsiveProps() {
        return m_propulsiveProps;
    }

    /**
     * Getter for the separator before the aerodynamic properties sub widget.
     * 
     * @return the separator before the aerodynamic properties sub widget
     */
    public GComponent getGSeparatorAeroProps() {
        return m_separators[3];
    }

    /**
     * Getter for the checkbox for the aerodynamic properties visibility.
     * 
     * @return the checkbox for the aerodynamic properties visibility.
     */
    public GCheckBoxWithIndicator getGCheckBoxAeroProps() {
        return m_aeroPropCheckbox;
    }

    /**
     * Getter for the widget to set the vehicle aerodynamic properties.
     * 
     * @return the widget to set the vehicle aerodynamic properties.
     */
    public GPVehicleAeroProps getGPVehicleAeroProps() {
        return m_aeroProps;
    }

    /**
     * Getter for the separator before the radiative properties sub widget.
     * 
     * @return the separator before the radiative properties sub widget
     */
    public GComponent getGSeparatorRadiativeProps() {
        return m_separators[4];
    }

    /**
     * Getter for the checkbox for the radiative properties visibility.
     * 
     * @return the checkbox for the radiative properties visibility.
     */
    public GCheckBoxWithIndicator getGCheckBoxRadiativeProps() {
        return m_radiativePropCheckbox;
    }

    /**
     * Getter for the widget to set the vehicle radiative properties.
     * 
     * @return the widget to set the vehicle radiative properties.
     */
    public GPVehicleRadiativeProps getGPVehicleRadiativeProps() {
        return m_radiativeProps;
    }

    /**
     * Getter for the different separators.
     * 
     * @param i separator index
     * @return the different separators.
     */
    public GSeparator getGEntryRealDragCoeff(final int i) {
        return m_separators[i];
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void generic() throws GException {
        beginOfElement(GPItemType.Vehicle, getNameInConfigFile());
        if (m_hasMassProps) {
            put(m_massProps);
        }

        if (m_hasPropulsiveProps) {
            put(m_propulsivePropCheckbox);
            if (m_propulsivePropCheckbox.isSelected()) {
                put(m_propulsiveProps);
            }
        }

        if (m_hasAeroProps) {
            put(m_aeroPropCheckbox);
            if (m_aeroPropCheckbox.isSelected()) {
                // Updating shape in aerodynamic properties
                m_aeroProps.setSurfaceModel(m_vehicleShape.getPatriusObject());
                put(m_aeroProps);
            }
        }

        if (m_hasRadiativeProps) {
            put(m_radiativePropCheckbox);
            if (m_radiativePropCheckbox.isSelected()) {
                // Shape not needed for radiative properties read/write operations.
                put(m_radiativeProps);
            }
        }

        if (hasAeroProperties() || hasRadiativeProperties()) {
            put(m_vehicleShape);
        }

        endOfElement();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void display() throws GException {
        put(m_label);

        if (m_massPropsVisible) {
            put(m_totalMassEntryReal);
        }
        put(m_separators[0]);

        if (m_hasMassProps && m_massPropsVisible) {
            put(m_massProps);
            put(m_separators[1]);
        }

        if (hasAeroProperties() || hasRadiativeProperties()) {
            put(m_vehicleShape);
            put(m_separators[2]);
        }

        if (m_hasPropulsiveProps) {
            put(m_propulsivePropCheckbox);
            if (m_propulsivePropCheckbox.isSelected()) {
                put(m_propulsiveProps);
            }
            put(m_separators[3]);
        }

        if (m_hasAeroProps) {
            put(m_aeroPropCheckbox);
            if (m_aeroPropCheckbox.isSelected()) {
                // Updating shape in aerodynamic properties
                m_aeroProps.setSurfaceModel(m_vehicleShape.getPatriusObject());
                put(m_aeroProps);
            }
            put(m_separators[4]);
        }

        if (m_hasRadiativeProps) {
            put(m_radiativePropCheckbox);
            if (m_radiativePropCheckbox.isSelected()) {
                // Shape not needed for radiative properties display operations.
                put(m_radiativeProps);
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void read() throws GException {
        generic();

        updateTotalMass();
        notifyObservers();
        // Update status and modified indicators
        updateStatus();
        updateIsModifiedIndicator();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void write() throws GException {
        generic();
        updateIsModifiedIndicator();
    }

    @Override
    public void clear() throws GException {
        if (m_hasMassProps) {
            put(m_massProps);
        }
        put(m_vehicleShape);
        if (m_hasPropulsiveProps) {
            put(m_propulsivePropCheckbox);
            put(m_propulsiveProps);
        }
        if (m_hasAeroProps) {
            put(m_aeroPropCheckbox);
            put(m_aeroProps);
        }
        if (m_hasRadiativeProps) {
            put(m_radiativePropCheckbox);
            put(m_radiativePropCheckbox);
        }

        updateTotalMass();

        // Update status and modified indicators
        updateStatus();
        updateIsModifiedIndicator();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void after(final GEvent e) throws GException {
        // Handling copy-paste event
        boolean handled;
        try {
            handled = handleCopyPasteEvent(m_label, e);
        } catch (final GFileManipulatorException exception) {
            throw new GPAttitudeException(exception);
        }
        // If it is a copy-paste event, exit after handling.
        if (handled) {
            return;
        }

        if (e.contains(m_vehicleShape)) {
            // Updating shape in aerodynamic and radiative
            m_aeroProps.setSurfaceModel(m_vehicleShape.getPatriusObject());
            // Shape not needed for radiative properties read/write/display operations.
            updateStatus();
            updateIsModifiedIndicator();
        }

        if (e.contains(m_massProps, m_propulsivePropCheckbox, m_propulsiveProps)) {
            updateTotalMass();
            updateStatus();
            updateIsModifiedIndicator();
        }

        if (e.contains(m_aeroPropCheckbox, m_aeroProps, m_radiativePropCheckbox, m_radiativeProps)) {
            updateStatus();
            updateIsModifiedIndicator();
        }

        notifyObservers();
    }

    /**
     * Method to update the total mas entry real as the sum of the dry mass and all fuel tanks
     * ergols masses.
     */
    private final void updateTotalMass() {
        try {
            if (hasMassProperties()) {
                // Has mass properties
                if (hasPropulsiveProperties()) {
                    // Has engine
                    m_totalMassEntryReal.setValue(m_massProps.getDryMass()
                            + m_propulsiveProps.getErgMass());
                } else {
                    m_totalMassEntryReal.setValue(m_massProps.getDryMass());
                }
            } else {
                // Does not have mass properties
                if (hasPropulsiveProperties()) {
                    // Has engine
                    m_totalMassEntryReal.setValue(m_propulsiveProps.getErgMass());
                } else {
                    m_totalMassEntryReal.setValue(0.0);
                }
            }
        } catch (final GArithmeticException e) {
            // They cannot be NaN comming from a GEntryReal
            throw new GRuntimeException(e);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    @SuppressWarnings("PMD.UnusedFormalParameter")
    public void before(final GEvent e) throws GException {
        // Nothing to do
    }

    /**
     * Boolean to know if mass properties are activated.
     * 
     * @return true if mass properties are activated.
     */
    public boolean hasMassProperties() {
        return m_hasMassProps;
    }

    /**
     * Boolean to know if propulsive properties are activated.
     * 
     * @return true if the vehicle has propulsive properties defined.
     */
    public boolean hasPropulsiveProperties() {
        return m_hasPropulsiveProps && m_propulsivePropCheckbox.isSelected();
    }

    /**
     * Boolean to know if aerodynamic properties are activated.
     * 
     * @return true if the vehicle has aerodynamic properties defined.
     */
    public boolean hasAeroProperties() {
        return m_hasAeroProps && m_aeroPropCheckbox.isSelected();
    }

    /**
     * Boolean to know if radiative properties are activated.
     * 
     * @return true if the vehicle has radiative properties defined.
     */
    public boolean hasRadiativeProperties() {
        return m_hasRadiativeProps && m_radiativePropCheckbox.isSelected();
    }

    /**
     * Get the mass property.
     * 
     * @return the mass property
     * @throws GPVehicleException if it has no mass properties defined or if the dry mass is
     *         negative.
     */
    public MassProperty getMassProperty() throws GPVehicleException {

        if (!hasMassProperties()) {
            throw new GPVehicleException(GEnvironment.trf(MISSING_PROPERTY_ERROR,
                    GEnvironment.tr("mass")));
        }

        final MassProperty massProperty = m_massProps.getPatriusObject();

        return massProperty;
    }

    /**
     * Get aerodynamic properties.
     * 
     * @return the aerodynamic properties
     * @throws GPVehicleException if it has no aerodynamic properties defined
     */
    public AerodynamicProperties getAerodynamicProperties() throws GPVehicleException {

        if (!m_hasAeroProps) {
            throw new GPVehicleException(GEnvironment.trf(MISSING_PROPERTY_ERROR,
                    GEnvironment.tr("aerodynamic")));
        }

        // Updating shape in aerodynamic properties
        m_aeroProps.setSurfaceModel(m_vehicleShape.getPatriusObject());
        // Aerodynamic properties
        final AerodynamicProperties aeroProperties = m_aeroProps.getPatriusObject();
        // Return aerodynamic properties
        return aeroProperties;
    }

    /**
     * Get radiative properties.
     * 
     * @return the radiative properties
     * @throws GPVehicleException if it has no radiative properties defined
     */
    public RadiativeProperties getRadiativeProperties() throws GPVehicleException {

        if (!m_hasRadiativeProps) {
            throw new GPVehicleException(GEnvironment.trf(MISSING_PROPERTY_ERROR,
                    GEnvironment.tr("radiative")));
        }

        // Updating shape in aerodynamic properties
        m_radiativeProps.setSurfaceModel(m_vehicleShape.getPatriusObject());
        // Radiative properties
        final RadiativeProperties radProperties = m_radiativeProps.getPatriusObject();
        // Return radiative properties
        return radProperties;
    }

    /**
     * Get the vehicle data.
     * 
     * @return the vehicle data.
     * @throws GPVehicleException if dry mass is negative
     */
    @Override
    public Vehicle getPatriusObject() throws GPVehicleException {

        // Vehicle data
        Vehicle vehicle = null;
        // Dry mass property
        MassProperty massProperty = null;
        // List of tanks
        ArrayList<TankProperty> listTanks = null;
        // List of engines
        ArrayList<PropulsiveProperty> listEngines = null;
        // Aerodynamic properties
        AerodynamicProperties aeroProperties = null;
        // Radiative properties
        RadiativeProperties radiativeProperties = null;

        // Dry mass property
        if (hasMassProperties()) {
            massProperty = getMassProperty();
        }

        // Propulsive properties
        if (hasPropulsiveProperties()) {
            listTanks = m_propulsiveProps.getTanksList();
            listEngines = m_propulsiveProps.getEnginesList();
        }

        // Aerodynamic properties
        if (hasAeroProperties()) {
            aeroProperties = getAerodynamicProperties();
        }

        // Radiative properties
        if (hasRadiativeProperties()) {
            radiativeProperties = getRadiativeProperties();
        }
        final CrossSectionProvider mainShape = m_vehicleShape.getPatriusObject();
        final List<Facet> solarPanels = new ArrayList<>();
        // Build vehicle data
        vehicle = new Vehicle(mainShape, solarPanels, massProperty, aeroProperties,
                radiativeProperties, listEngines, listTanks);

        // Return
        return vehicle;

    }

    /**
     * {@inheritDoc}
     * 
     * @param args The parameters are 0: ExtendedOneAxisEllipsoid 1: Atmosphere.
     */
    @Override
    public Vehicle getPatriusObject(final Object... args) throws GException {
        // Check the number of input arguments and their classes
        new ArgsChecker(args, ExtendedOneAxisEllipsoid.class, Atmosphere.class);

        final ExtendedOneAxisEllipsoid prevEarthShape = getEarthShape();
        setEarthShape((ExtendedOneAxisEllipsoid) args[0]);

        final Atmosphere prevAtmosphere = getAtmosphere();
        setAtmosphere((Atmosphere) args[1]);

        final Vehicle patriusObject = getPatriusObject();

        setEarthShape(prevEarthShape);
        setAtmosphere(prevAtmosphere);

        return patriusObject;
    }

    /**
     * Get vehicle assembly.
     * 
     * @param propagationFrame propagation frame
     * @return the vehicle assembly
     * @throws GPVehicleException if dry mass is negative
     * @throws PatriusException problem to the assembly creation
     */
    public Assembly getAssembly(final Frame propagationFrame) throws GPVehicleException, PatriusException {
        return getPatriusObject().createAssembly(propagationFrame);
    }

    /**
     * Update checkbox status for underlying components.
     */
    private void updateCheckboxStatus() {
        // Propulsive properties
        if (m_hasPropulsiveProps) {
            if (m_propulsivePropCheckbox.isSelected()) {
                m_propulsivePropCheckbox.getGLabelWithIndicator().setStatus(
                        m_propulsiveProps.getStatus());
            } else {
                m_propulsivePropCheckbox.getGLabelWithIndicator().setStatus(GStatus.OK);
            }
        }
        // Aero properties
        if (m_hasAeroProps) {
            if (m_aeroPropCheckbox.isSelected()) {
                m_aeroPropCheckbox.getGLabelWithIndicator().setStatus(m_aeroProps.getStatus());
            } else {
                // Forced status case because the GPForceModels requires it
                final GLabelWithIndicator label = m_aeroPropCheckbox.getGLabelWithIndicator();
                if (label.isForcedStatus() != null) {
                    label.setStatus(label.isForcedStatus());
                    label.setToolTipText(GEnvironment.tr("Error: the force model requires aerodynamic properties"));
                } else {
                    label.setStatus(GStatus.OK);
                    label.setToolTipText("");
                }
            }
        }
        // Radiative properties
        if (m_hasRadiativeProps) {
            if (m_radiativePropCheckbox.isSelected()) {
                m_radiativePropCheckbox.getGLabelWithIndicator().setStatus(m_radiativeProps.getStatus());
            } else {
                // Forced status case
                final GLabelWithIndicator label = m_radiativePropCheckbox.getGLabelWithIndicator();
                if (label.isForcedStatus() != null) {
                    label.setStatus(label.isForcedStatus());
                    label.setToolTipText(GEnvironment.tr("Error: the force model requires radiative properties"));
                } else {
                    label.setStatus(GStatus.OK);
                    label.setToolTipText("");
                }
            }
        }
    }
    
    /**
     * Update mass status.
     */
    private void updateMassStatus() {
        if (hasMassProperties()) {
            final double totalMass = m_totalMassEntryReal.getValue();
            final GLabelWithIndicator label = m_totalMassEntryReal.getGLabelWithIndicator();
            if (totalMass == 0 && label.isForcedStatus() != null) {
                m_massProps.getGEntryRealDryMass().getGLabelWithIndicator().setStatus(label.isForcedStatus());
                m_massProps.getGEntryRealDryMass().getGField().getJTextField()
                        .setToolTipText(GEnvironment.tr("Error: the force model requires mass properties"));
            } else {
                m_massProps.getGEntryRealDryMass().getGLabelWithIndicator()
                        .setStatus(m_massProps.getGEntryRealDryMass().getStatus());
                m_massProps.getGEntryRealDryMass().updateStatus();
            }
        }
    }

    /**
     * Update the global status of the widget following the status of its components.
     */
    private final void updateStatus() {

        // Updating status of the checkboxes
        updateCheckboxStatus();
        
        // Update mass status
        updateMassStatus();

        // Compute global status
        GStatus newStatus = null;
        String newToolTip = null;

        final GCondensedStatus condensedStatus = new GCondensedStatus(this);
        newStatus = condensedStatus.getStatus();

        // Setting tooltip message indicating which component is in error status
        final String COMMA = ", ";
        switch (newStatus) {
            case OK:
                // Removing label tooltip text
                newToolTip = null;
                break;

            case WARN:

                // Setting warning components list
                final StringBuffer warningBuffer = new StringBuffer();
                boolean firstTime = true;
                for (final GComponent component : condensedStatus.getWarningComponentList()) {
                    if (firstTime) {
                        // Avoid to put a comma in first place
                        firstTime = false;
                    } else {
                        warningBuffer.append(COMMA);
                    }
                    warningBuffer.append(component.getNameInConfigFile());
                }
                final String warningList = warningBuffer.toString();

                // List of components in warning status
                newToolTip = GEnvironment.getWarningTooltipMessage(warningList);
                // Log event
                GEnvironment.getGLog().writeLogWarn(Type.OPER, getWidgetId(),
                        GEnvironment.tr("Value in warning range."));
                break;

            case ERROR:

                // Setting error components list
                final StringBuffer errorBuffer = new StringBuffer();
                firstTime = true;
                for (final GComponent component : condensedStatus.getErrorComponentList()) {
                    if (firstTime) {
                        // Avoid to put a comma in first place
                        firstTime = false;
                    } else {
                        errorBuffer.append(COMMA);
                    }
                    errorBuffer.append(component.getNameInConfigFile());
                }
                final String errorList = errorBuffer.toString();

                // List of components in error status
                newToolTip = GEnvironment.getErrorTooltipMessage(errorList);
                // Log event
                GEnvironment.getGLog().writeLogWarn(Type.OPER, getWidgetId(),
                        GEnvironment.tr("Value in error range."));
                break;
        }
        setStatus(newStatus);
        getGLabel().setStatus(newStatus);
        getGLabel().setToolTipText(newToolTip);
    }

    /** {@inheritDoc} */
    @Override
    public void updateCondensedStatus(final GCondensedStatus condensedStatus) {
        condensedStatus.update(m_vehicleShape);
        // Update status only for visible properties
        if (hasMassProperties()) {
            condensedStatus.update(m_massProps);
        }
        if (hasPropulsiveProperties()) {
            condensedStatus.update(m_propulsiveProps);
        }
        if (hasAeroProperties()) {
            condensedStatus.update(m_aeroProps);
        }
        if (hasRadiativeProperties()) {
            condensedStatus.update(m_radiativeProps);
        }
        // Specific case for Vehicle/GPForceModels link
        // Mass properties
        if (m_massProps != null) {
            condensedStatus.update(m_massProps.getGEntryRealDryMass().getGLabelWithIndicator());
        }
        // Aero properties
        if (m_aeroPropCheckbox != null) {
            condensedStatus.update(m_aeroPropCheckbox.getGLabelWithIndicator());
        }
        // Radiative properties
        if (m_radiativePropCheckbox != null) {
            condensedStatus.update(m_radiativePropCheckbox.getGLabelWithIndicator());
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isEnabled() {
        return m_label.isEnabled();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setEnabled(final boolean b) {
        m_label.setEnabled(b);
        if (m_hasMassProps) {
            m_massProps.setEnabled(b);
        }
        // Total mass is allways disabled
        m_vehicleShape.setEnabled(b);
        if (m_hasPropulsiveProps) {
            m_propulsivePropCheckbox.setEnabled(b);
            m_propulsiveProps.setEnabled(b);
        }
        if (m_hasAeroProps) {
            m_aeroPropCheckbox.setEnabled(b);
            m_aeroProps.setEnabled(b);
        }
        if (m_hasRadiativeProps) {
            m_radiativePropCheckbox.setEnabled(b);
            m_radiativeProps.setEnabled(b);
        }
        for (int i = 0; i < m_separators.length; i++) {
            m_separators[i].setEnabled(b);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isModified() {

        // Initialization
        boolean modified = m_vehicleShape.isModified();

        if (m_hasMassProps) {
            // Mass properties
            modified = modified || m_massProps.isModified();
        }
        if (m_hasPropulsiveProps) {
            // Engine properties
            modified = modified || m_propulsivePropCheckbox.isModified();
            if (m_propulsivePropCheckbox.isSelected()) {
                modified = modified || m_propulsiveProps.isModified();
            }
        }
        if (m_hasAeroProps) {
            // Aero properties
            modified = modified || m_aeroPropCheckbox.isModified();
            if (m_aeroPropCheckbox.isSelected()) {
                modified = modified || m_aeroProps.isModified();
            }
        }
        if (m_hasRadiativeProps) {
            // Radiative properties
            modified = modified || m_radiativePropCheckbox.isModified();
            if (m_radiativePropCheckbox.isSelected()) {
                modified = modified || m_radiativeProps.isModified();
            }
        }

        // Return result
        return modified;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void updateIsModifiedIndicator() {
        if (m_hasPropulsiveProps) {
            // Engine properties
            if (m_propulsivePropCheckbox.isModified()) {
                m_propulsiveProps.setDisplayIsModifiedIndicatorAll(DisplayIndicatorStatus.Always);
            } else {
                m_propulsiveProps
                        .setDisplayIsModifiedIndicatorAll(DisplayIndicatorStatus.Automatic);
            }
        }
        if (m_hasAeroProps) {
            // Aero properties
            if (m_aeroPropCheckbox.isModified()) {
                m_aeroProps.setDisplayIsModifiedIndicatorAll(DisplayIndicatorStatus.Always);
            } else {
                m_aeroProps.setDisplayIsModifiedIndicatorAll(DisplayIndicatorStatus.Automatic);
            }
        }
        if (m_hasRadiativeProps) {
            // Radiative properties
            if (m_radiativePropCheckbox.isModified()) {
                m_radiativeProps.setDisplayIsModifiedIndicatorAll(DisplayIndicatorStatus.Always);
            } else {
                m_radiativeProps.setDisplayIsModifiedIndicatorAll(DisplayIndicatorStatus.Automatic);
            }
        }
        // Finalization
        m_label.setIndicatorVisible(getDisplayIsModifiedIndicator());
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public GComponent getSubComponent(final int position) throws GComponentException {
        GComponent subcomponent = null;
        final ArrayList<GComponent> listOfNonNullComps = new ArrayList<GComponent>();
        listOfNonNullComps.add(m_label);
        listOfNonNullComps.add(m_totalMassEntryReal);
        if (m_hasMassProps) {
            listOfNonNullComps.add(m_massProps);
        }
        listOfNonNullComps.add(m_vehicleShape);
        listOfNonNullComps.add(m_propulsivePropCheckbox);
        if (m_hasPropulsiveProps) {
            listOfNonNullComps.add(m_propulsiveProps);
        }
        listOfNonNullComps.add(m_aeroPropCheckbox);
        if (m_hasAeroProps) {
            listOfNonNullComps.add(m_aeroProps);
        }
        listOfNonNullComps.add(m_radiativePropCheckbox);
        if (m_hasRadiativeProps) {
            listOfNonNullComps.add(m_radiativeProps);
        }
        if (position < listOfNonNullComps.size()) {
            subcomponent = listOfNonNullComps.get(position);
        } else {
            launchInvalidPositionException(position);
        }
        return subcomponent;
    }

    /**
     * Method to get the list of engine names.
     * 
     * @return list of engine names
     * @throws GPVehicleException if propulsive features were not defined
     */
    public String[] getEngineNames() throws GPVehicleException {

        // Check propulsive features
        if (!hasPropulsiveProperties()) {
            throw new GPVehicleException(GEnvironment.trf(MISSING_PROPERTY_ERROR,
                    GEnvironment.tr(PROPULSIVE_STR)));
        }

        final String[] nameslist = m_propulsiveProps.getEngineNames();

        // Return
        return nameslist;
    }

    /**
     * Method to get the list of tank names.
     * 
     * @return list of tank names
     * @throws GPVehicleException if propulsive features were not defined
     */
    public String[] getTankNames() throws GPVehicleException {

        if (!hasPropulsiveProperties()) {
            throw new GPVehicleException(GEnvironment.trf(MISSING_PROPERTY_ERROR,
                    GEnvironment.tr(PROPULSIVE_STR)));
        }

        final String[] nameslist = m_propulsiveProps.getTankNames();

        return nameslist;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public ArrayList<TankProperty> getFuelTankList() {
        ArrayList<TankProperty> tanksList = new ArrayList<TankProperty>();
        if (hasPropulsiveProperties()) {
            // Get list of tanks
            tanksList = m_propulsiveProps.getTanksList();
        }
        return tanksList;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public ArrayList<PropulsiveProperty> getEngineList() {
        ArrayList<PropulsiveProperty> enginesList = new ArrayList<PropulsiveProperty>();
        if (hasPropulsiveProperties()) {
            // Get list of engines
            enginesList = m_propulsiveProps.getEnginesList();
        }
        return enginesList;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void registerObserver(final GObserver observer) {
        m_observerList.registerObserver(observer);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void unregisterObserver(final GObserver observer) {
        m_observerList.unregisterObserver(observer);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void notifyObservers(final Object... args) {
        m_observerList.notifyObservers(this, args);
    }

    /**
     * Setter for the vehicle.
     * 
     * @param vehicle vehicle to be set
     * @throws GPVehicleException if there is any kind of incompatibility between the input vehicle
     *         and the widget.
     * @throws GArithmeticException if dryMass is NaN, or<br>
     *         if vehicleShape has sphere, parallelepiped, cylinder or solar panels surfaces equal
     *         to NaN, or<br>
     *         if aero coeffs are constant and NaN dans aeroProp, or<br>
     *         if any of the radiative coefficients in radiativeProperty is NaN.
     */
    public final void setVehicle(final Vehicle vehicle) throws GPVehicleException, GArithmeticException {
        // Mass property
        if (m_hasMassProps) {
            if (vehicle.getMassProperty() == null) {
                m_massProps.setDryMass(0.0);
            } else {
                m_massProps.setDryMass(vehicle.getDryMass());
            }
        } else {
            if (vehicle.getMassProperty() != null) {
                throw new GPVehicleException(String.format(
                        GEnvironment.tr(INCOMPAT_WITH_WIDGET_ERROR),
                        GEnvironment.tr("a mass property")));
            }
        }

        // Vehicle shape
        final VehicleSurfaceModel vehShape = getShapeFromVeh(vehicle);
        m_vehicleShape.setVehicleShape(vehShape);

        // Propulsive properties
        if (m_hasPropulsiveProps) {
            if (vehicle.getEnginesList().isEmpty() && vehicle.getTanksList().isEmpty()) {
                m_propulsivePropCheckbox.setSelected(false);
            } else {
                m_propulsivePropCheckbox.setSelected(true);
                if (!vehicle.getEnginesList().isEmpty()) {
                    m_propulsiveProps.setEnginesList(vehicle.getEnginesList());
                }
                if (!vehicle.getTanksList().isEmpty()) {
                    m_propulsiveProps.setTanksList(vehicle.getTanksList());
                }
            }
        } else {
            if (!vehicle.getEnginesList().isEmpty() || !vehicle.getTanksList().isEmpty()) {
                throw new GPVehicleException(String.format(
                        GEnvironment.tr(INCOMPAT_WITH_WIDGET_ERROR),
                        GEnvironment.tr("propulsive properties")));
            }
        }

        // Aerodynamic properties
        if (m_hasAeroProps) {
            if (vehicle.getAerodynamicProperties() == null) {
                m_aeroPropCheckbox.setSelected(false);
            } else {
                m_aeroPropCheckbox.setSelected(true);
                try {
                    m_aeroProps.setAerodynamicProperties(vehicle.getAerodynamicProperties());
                } catch (final PatriusException e) {
                    throw new GPVehicleException(String.format(GEnvironment
                            .tr(INCOMPATIBLE_AERO_PROPERTIES)));
                }
            }
        } else {
            if (vehicle.getAerodynamicProperties() != null) {
                throw new GPVehicleException(String.format(
                        GEnvironment.tr(INCOMPAT_WITH_WIDGET_ERROR),
                        GEnvironment.tr("aerodynamic properties")));
            }
        }

        // Radiative properties
        if (m_hasRadiativeProps) {
            if (vehicle.getRadiativeProperties() == null) {
                m_radiativePropCheckbox.setSelected(false);
            } else {
                m_radiativePropCheckbox.setSelected(true);
                m_radiativeProps.setRadiativeProperties(vehicle.getRadiativeProperties());
            }
        } else {
            if (vehicle.getRadiativeProperties() != null) {
                throw new GPVehicleException(String.format(
                        GEnvironment.tr(INCOMPAT_WITH_WIDGET_ERROR),
                        GEnvironment.tr("radiative properties")));
            }
        }

        updateTotalMass();
        updateStatus();
        updateIsModifiedIndicator();
    }

    /**
     * Checks the combination of shapes in aerodynamic and radiative properties and compares with
     * the widget to get the shape to be applied.
     * 
     * @param vehicle from which extract the shape.
     * @return the shape to be inserted in the widget
     * @throws GPVehicleException if the widget is not compatible with the input vehicle
     */
    private VehicleSurfaceModel getShapeFromVeh(final Vehicle vehicle) throws GPVehicleException {
        VehicleSurfaceModel shapeFromAeroProps = null;
        if (vehicle.getAerodynamicProperties() != null) {
            shapeFromAeroProps = vehicle.getAerodynamicProperties().getVehicleSurfaceModel();
        }
        VehicleSurfaceModel shapeFromRadProps = null;
        if (vehicle.getRadiativeProperties() != null) {
            shapeFromRadProps = vehicle.getRadiativeProperties().getVehicleSurfaceModel();
        }
        VehicleSurfaceModel nullShape = null;
        try {
            nullShape = new VehicleSurfaceModel(new Sphere(0));
        } catch (final PatriusException e) {
            // It cannot happen
            throw new PatriusRuntimeException(e.getMessage(), e);
        }
        VehicleSurfaceModel vehShape = null;
        if (m_hasAeroProps && m_hasPropulsiveProps) {
            if (shapeFromAeroProps == null && shapeFromRadProps == null) {
                vehShape = nullShape;
            } else if (shapeFromAeroProps == null && shapeFromRadProps != null) {
                vehShape = shapeFromRadProps;
            } else if (shapeFromAeroProps != null && shapeFromRadProps == null) {
                vehShape = shapeFromAeroProps;
            } else if (shapeFromAeroProps != null && shapeFromAeroProps.equals(shapeFromRadProps)) {
                vehShape = shapeFromAeroProps;
            } else {
                throw new GPVehicleException(
                        // MSG_STATEMENT
                        GEnvironment
                                .tr("Different vehicle shapes for aerodynamic properties and radiative properties."));
            }
        } else if (m_hasAeroProps && !m_hasPropulsiveProps) {
            if (shapeFromAeroProps == null) {
                vehShape = nullShape;
            } else {
                vehShape = shapeFromAeroProps;
            }
        } else if (!m_hasAeroProps && m_hasPropulsiveProps) {
            if (shapeFromRadProps == null) {
                vehShape = nullShape;
            } else {
                vehShape = shapeFromRadProps;
            }
        } else {
            vehShape = nullShape;
        }
        return vehShape;
    }

    /**
     * Setter for the default vehicle.
     * 
     * @param vehicle the default vehicle to be set
     * @throws GPVehicleException if there is any kind of incompatibility between the input vehicle
     *         and the widget.
     * @throws GArithmeticException if dryMass is NaN, or<br>
     *         if vehicleShape has sphere, parallelepiped, cylinder or solar panels surfaces equal
     *         to NaN, or<br>
     *         if aero coeffs are constant and NaN dans aeroProp, or<br>
     *         if any of the radiative coefficients in radiativeProperty is NaN.
     */
    public final void setDefaultVehicle(final Vehicle vehicle) throws GPVehicleException, GArithmeticException {

        // Mass property
        if (m_hasMassProps) {
            if (vehicle.getMassProperty() == null) {
                m_massProps.setDefaultDryMass(0.0);
            } else {
                m_massProps.setDefaultDryMass(vehicle.getDryMass());
            }
        } else {
            if (vehicle.getMassProperty() != null) {
                throw new GPVehicleException(String.format(
                        GEnvironment.tr(INCOMPAT_WITH_WIDGET_ERROR),
                        GEnvironment.tr("a default mass property")));
            }
        }

        // Vehicle shape
        final VehicleSurfaceModel vehShape = getShapeFromVeh(vehicle);
        m_vehicleShape.setDefaultVehicleShape(vehShape);

        // Propulsive properties
        if (m_hasPropulsiveProps) {
            if (vehicle.getEnginesList().isEmpty() && vehicle.getTanksList().isEmpty()) {
                m_propulsivePropCheckbox.setDefaultValue(false);
            } else {
                m_propulsivePropCheckbox.setDefaultValue(true);
                if (!vehicle.getEnginesList().isEmpty()) {
                    m_propulsiveProps.setDefaultEnginesList(vehicle.getEnginesList());
                }
                if (!vehicle.getTanksList().isEmpty()) {
                    m_propulsiveProps.setDefaultTanksList(vehicle.getTanksList());
                }
            }
        } else {
            if (!vehicle.getEnginesList().isEmpty() || !vehicle.getTanksList().isEmpty()) {
                throw new GPVehicleException(String.format(
                        GEnvironment.tr(INCOMPAT_WITH_WIDGET_ERROR),
                        GEnvironment.tr("default propulsive properties")));
            }
        }

        // Aerodynamic properties
        if (m_hasAeroProps) {
            if (vehicle.getAerodynamicProperties() == null) {
                m_aeroPropCheckbox.setDefaultValue(false);
            } else {
                m_aeroPropCheckbox.setDefaultValue(true);
                try {
                    m_aeroProps.setDefaultAerodynamicProperties(vehicle.getAerodynamicProperties());
                } catch (final PatriusException e) {
                    throw new GPVehicleException(String.format(GEnvironment
                            .tr(INCOMPATIBLE_AERO_PROPERTIES)));
                }
            }
        } else {
            if (vehicle.getAerodynamicProperties() != null) {
                throw new GPVehicleException(String.format(
                        GEnvironment.tr(INCOMPAT_WITH_WIDGET_ERROR),
                        GEnvironment.tr("default aerodynamic properties")));
            }
        }

        // Radiative properties
        if (m_hasRadiativeProps) {
            if (vehicle.getRadiativeProperties() == null) {
                m_radiativePropCheckbox.setDefaultValue(false);
            } else {
                m_radiativePropCheckbox.setDefaultValue(true);
                m_radiativeProps.setDefaultRadiativeProperties(vehicle.getRadiativeProperties());
            }
        } else {
            if (vehicle.getRadiativeProperties() != null) {
                throw new GPVehicleException(String.format(
                        GEnvironment.tr(INCOMPAT_WITH_WIDGET_ERROR),
                        GEnvironment.tr("default radiative properties")));
            }
        }
    }

    /**
     * Setter for the saved vehicle.
     * 
     * @param vehicle the saved vehicle to be set
     * @throws GPVehicleException if there is any kind of incompatibility between the input vehicle
     *         and the widget.
     * @throws GArithmeticException if dryMass is NaN, or<br>
     *         if vehicleShape has sphere, parallelepiped, cylinder or solar panels surfaces equal
     *         to NaN, or<br>
     *         if aero coeffs are constant and NaN dans aeroProp, or<br>
     *         if any of the radiative coefficients in radiativeProperty is NaN.
     */
    public final void setSavedVehicle(final Vehicle vehicle) throws GPVehicleException, GArithmeticException {

        // Mass property
        if (m_hasMassProps) {
            if (vehicle.getMassProperty() == null) {
                m_massProps.setSavedDryMass(0.0);
            } else {
                m_massProps.setSavedDryMass(vehicle.getDryMass());
            }
        } else {
            if (vehicle.getMassProperty() != null) {
                throw new GPVehicleException(String.format(
                        GEnvironment.tr(INCOMPAT_WITH_WIDGET_ERROR),
                        GEnvironment.tr("a saved mass property")));
            }
        }

        // Vehicle shape
        final VehicleSurfaceModel vehShape = getShapeFromVeh(vehicle);
        m_vehicleShape.setSavedVehicleShape(vehShape);

        // Propulsive properties
        if (m_hasPropulsiveProps) {
            if (vehicle.getEnginesList().isEmpty() && vehicle.getTanksList().isEmpty()) {
                m_propulsivePropCheckbox.setSavedValue(false);
            } else {
                m_propulsivePropCheckbox.setSavedValue(true);
                if (!vehicle.getEnginesList().isEmpty()) {
                    m_propulsiveProps.setSavedEnginesList(vehicle.getEnginesList());
                }
                if (!vehicle.getTanksList().isEmpty()) {
                    m_propulsiveProps.setSavedTanksList(vehicle.getTanksList());
                }
            }
        } else {
            if (!vehicle.getEnginesList().isEmpty() || !vehicle.getTanksList().isEmpty()) {
                throw new GPVehicleException(String.format(
                        GEnvironment.tr(INCOMPAT_WITH_WIDGET_ERROR),
                        GEnvironment.tr("saved propulsive properties")));
            }
        }

        // Aerodynamic properties
        if (m_hasAeroProps) {
            if (vehicle.getAerodynamicProperties() == null) {
                m_aeroPropCheckbox.setSavedValue(false);
            } else {
                m_aeroPropCheckbox.setSavedValue(true);
                try {
                    m_aeroProps.setSavedAerodynamicProperties(vehicle.getAerodynamicProperties());
                } catch (final PatriusException e) {
                    throw new GPVehicleException(String.format(GEnvironment
                            .tr(INCOMPATIBLE_AERO_PROPERTIES)));
                }
            }
        } else {
            if (vehicle.getAerodynamicProperties() != null) {
                throw new GPVehicleException(String.format(
                        GEnvironment.tr(INCOMPAT_WITH_WIDGET_ERROR),
                        GEnvironment.tr("saved aerodynamic properties")));
            }
        }

        // Radiative properties
        if (m_hasRadiativeProps) {
            if (vehicle.getRadiativeProperties() == null) {
                m_radiativePropCheckbox.setSavedValue(false);
            } else {
                m_radiativePropCheckbox.setSavedValue(true);
                m_radiativeProps.setSavedRadiativeProperties(vehicle.getRadiativeProperties());
            }
        } else {
            if (vehicle.getRadiativeProperties() != null) {
                throw new GPVehicleException(String.format(
                        GEnvironment.tr(INCOMPAT_WITH_WIDGET_ERROR),
                        GEnvironment.tr("saved radiative properties")));
            }
        }

        updateIsModifiedIndicator();
    }

    /**
     * Setter for the Earth shape.
     * 
     * @param earthShape the Earth's shape
     * @throws GArithmeticException if equatorial radius or flatness in earthShape are NaN
     */
    public void setEarthShape(final ExtendedOneAxisEllipsoid earthShape) throws GArithmeticException {
        if (m_hasAeroProps) {
            m_aeroProps.setEarthShape(earthShape);
        }
    }

    /**
     * Getter for the Earth shape.
     * 
     * @return the Earth's shape
     */
    public ExtendedOneAxisEllipsoid getEarthShape() {
        ExtendedOneAxisEllipsoid earthShape = null;
        if (m_hasAeroProps) {
            earthShape = m_aeroProps.getEarthShape();
        }
        return earthShape;
    }

    /**
     * Setter for the atmosphere.
     * 
     * @param atmosphere the atmosphere
     */
    public void setAtmosphere(final Atmosphere atmosphere) {
        if (m_hasAeroProps) {
            m_aeroProps.setAtmosphere(atmosphere);
        }
    }

    /**
     * Getter for the atmosphere.
     * 
     * @return the atmosphere
     */
    public Atmosphere getAtmosphere() {
        Atmosphere atmosphere = null;
        if (m_hasAeroProps) {
            atmosphere = m_aeroProps.getAtmosphere();
        }
        return atmosphere;
    }

    /**
     * Sets the mass properties and the total mass visible / invisible.
     * 
     * @param visible visible if true; invisible if false
     */
    public void setMassPropertiesVisible(final boolean visible) {
        m_massPropsVisible = visible;
    }

    /** {@inheritDoc} */
    @Override
    public void notify(final Object observable, final Object... args) {
    	
        //JFG DEB MOD
    	if ( observable instanceof GPForceModelsLocal) {
    		
            // Retrieve the observable which is a GPForceModels
            // Update GPVehicle status depending on user force models
            final GPForceModelsLocal forceModels = (GPForceModelsLocal) observable;
            
            // Drag force
            if (m_hasAeroProps) {
                if (forceModels.isAeroForceModelActive()) {
                    m_aeroPropCheckbox.getGLabelWithIndicator().setForcedStatus(GStatus.ERROR);
                } else {
                    m_aeroPropCheckbox.getGLabelWithIndicator().setForcedStatus(null);
                }
            }
            
            // SRP force and rediffused SRP force
            if (m_hasRadiativeProps) {
                if (forceModels.isSRPForceModelActive() || forceModels.isRediffusedSRPForceModelActive()) {
                    m_radiativePropCheckbox.getGLabelWithIndicator().setForcedStatus(GStatus.ERROR);
                } else {
                    m_radiativePropCheckbox.getGLabelWithIndicator().setForcedStatus(null);
                }
            }

            if (m_hasAeroProps || m_hasRadiativeProps) {
                if (forceModels.isAeroForceModelActive() || forceModels.isSRPForceModelActive() || forceModels.isRediffusedSRPForceModelActive()) {
                    m_totalMassEntryReal.getGLabelWithIndicator().setForcedStatus(GStatus.ERROR);
                } else {
                    m_totalMassEntryReal.getGLabelWithIndicator().setForcedStatus(null);
                }
            }

            updateStatus();

    	}
        //JFG END MOD

    }
}
